/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2016 OX Software GmbH.
 *
 * @author Daniel Rentz <daniel.rentz@open-xchange.com>
 */

define('io.ox/office/editframework/view/control/borderwidthpicker', [
    'io.ox/office/tk/utils',
    'io.ox/office/tk/control/combofield',
    'io.ox/office/editframework/view/editlabels',
    'gettext!io.ox/office/editframework'
], function (Utils, ComboField, EditLabels, gt) {

    'use strict';

    // class BorderWidthPicker ================================================

    /**
     * A combo field with predefined entries for border line widths, in points.
     *
     * @constructor
     *
     * @extends ComboField
     *
     * @param {Object} [initOptions]
     *  Optional parameters. Supports all options supported by the base class
     *  ComboField.
     */
    function BorderWidthPicker(initOptions) {

        // base constructor ---------------------------------------------------

        initOptions = Utils.extendOptions({
            width: 40,
            tooltip: /*#. line width of borders in paragraphs and tables cells */ gt.pgettext('borders', 'Border width'),
            style: 'text-align:right;',
            keyboard: 'number',
            validator: new ComboField.NumberValidator({ min: 0.5, max: 10, precision: 0.1 })
        }, initOptions);
        ComboField.call(this, Utils.extendOptions({
            dropDownVersion: { label: Utils.getStringOption(initOptions, 'tooltip') }
        }, initOptions));

        // initialization -----------------------------------------------------

        _.each([0.5, 1, 1.5, 2, 2.5, 3, 4, 6], function (size) {
            this.createListEntry(size, {
                label: gt.format(
                    //#. A single list entry of a GUI border width picker.
                    //#. Example result: "5 points"
                    //#. %1$d is the width of the border line.
                    //#, c-format
                    gt.npgettext('borders', '%1$d point', '%1$d points', size),
                    _.noI18n(size)
                ),
                style: 'text-align:center;'
            });
        }, this);

    } // class BorderWidthPicker

    // exports ================================================================

    // derive this class from class ComboField
    return ComboField.extend({ constructor: BorderWidthPicker });

});
