/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2016 OX Software GmbH.
 *
 * @author Daniel Rentz <daniel.rentz@open-xchange.com>
 */

define('io.ox/office/spreadsheet/view/control/formatcategorypicker', [
    'io.ox/office/tk/utils',
    'io.ox/office/tk/control/radiolist',
    'io.ox/office/spreadsheet/view/labels',
    'gettext!io.ox/office/spreadsheet'
], function (Utils, RadioList, Labels, gt) {

    'use strict';

    var // all predefined number format categories to be inserted into the list
        FORMAT_CATEGORIES = ['standard', 'number', 'currency', 'date', 'time', 'datetime', 'percent', 'text', 'scientific', 'fraction'];

    // class FormatCategoryPicker =============================================

    /**
     * A drop-down list control for choosing number format categories, that
     * will load predefined format codes in the FormatCodePicker.
     *
     * @constructor
     *
     * @extends RadioList
     *
     * @param {Object} [initOptions]
     *  Optional parameters. Supports all options of the base class RadioList.
     */
    function FormatCategoryPicker(app, initOptions) {

        var // self reference
            self = this;

        // base constructor ---------------------------------------------------

        RadioList.call(this, Utils.extendOptions({
            label: _.noI18n('123'),
            tooltip: gt('Number format')
        }, initOptions, {
            updateCaptionMode: 'none'
        }));

        // initialization -----------------------------------------------------

        this.getNode().addClass('format-category-group');

        // initialize the drop-down list items
        this.listenTo(app.getImportPromise(), 'done', function () {
            // insert all supported list items
            _.each(FORMAT_CATEGORIES, function (category) {
                self.createOptionButton(category, { label: Labels.FORMAT_CATEGORY_LABELS[category] });
            });
        });

        // destroy all class members on destruction
        this.registerDestructor(function () {
            app = initOptions = self = null;
        });

    } // class FormatCategoryPicker

    // exports ================================================================

    // derive this class from class RadioList
    return RadioList.extend({ constructor: FormatCategoryPicker });

});
